<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use HCM\AVL\Services\BancoQuestoesService;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class BancoQuestoesController extends Crud {

    protected $BancoQuestoesService;
    protected $entityManager;
    protected $dataSourceName = 'avl_bancoquestoes';
    protected $dataSourceNameGet = 'getQuestoes';
    protected $dataSourceNameAvaliacoes = 'avl_avaliacoes';
    protected $dataSourceNameAvaliacoesGet = 'getAvaliacoes';
    protected $dataSourceNameAvaliacoesSelect = 'avl_avaliacoesselect';
    protected $dataSourceNameAvaliacoesSelectGet = 'getAvaliacoesSelect';
    protected $dataSourceNameRespostas = 'avl_bancorespostas';
    protected $dataSourceNameRespostasGet = 'getRespostas';
    protected $dataSourceNameNiveis = 'avl_banconiveis';
    protected $dataSourceNameNiveisGet = 'getNiveis';
    protected $dataSourceNameNAQ = 'avl_bancoquestoesNAQ';
    protected $dataSourceNameNAQGet = 'getNivelQuestaoAvaliacao';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
   
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, BancoQuestoesService $bancoQuestoesService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->BancoQuestoesService = $bancoQuestoesService;
        parent::__construct($dataSourceManager);
    }
   
    //FUNÇÕES DOS TIPOS DE AVALIÇÃO USADO POR QUESTÕES

    public function getAvaliacoesSelect(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRQUESTAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameAvaliacoesSelectGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getAvaliacoes(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRQUESTAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameAvaliacoesGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setAvaliacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRQUESTIPOAVAL', $row)) {
                    if ($row['__is_new']) {
                        $row['NRQUESTIPOAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTIPOAVAL', 1, 22, true); 
                    }
                } else {
                    $row['NRQUESTIPOAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTIPOAVAL', 1, 22, true); 
                }
            
            }
            $this->dataSourceName = $this->dataSourceNameAvaliacoes;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameAvaliacoes;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    //FUNCÕES DAS QUESTÕES 

    public function getQuestoes(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setQuestao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                if (array_key_exists('NRQUESTAO', $row)) {
                    if ($row['__is_new']) {
                        $row['NRQUESTAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTAO', 1, 22, true); 
                    }
                } else {
                    $row['NRQUESTAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTAO', 1, 22, true); 
                }
            
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteQuestao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $registros = $this->BancoQuestoesService->deleteQuestao($dataset);
            if($registros = true){
                parent::delete($request, $response);
            }
        } catch(\Exception $e) {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        
        }   
    }


    //FUNCÕES DAS RESPOSTAS
    
    public function getRespostas(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameRespostasGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setResposta(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRESPOSTA', $row)) {
                    if ($row['__is_new']) {
                        $row['NRRESPOSTA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'RESPOSTA', 1, 22, true); 
                    }
                } else {
                    $row['NRRESPOSTA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'RESPOSTA', 1, 22, true); 
                }
            
            }
            $this->dataSourceName = $this->dataSourceNameRespostas;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteResposta(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameRespostas;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
        $this->dataSourceName = $this->dataSourceNameRespostas;   
    }


    //FUNÇÕES DOS NÍVEIS DE QUESTÃO POR NÍVEL DE AVALIAÇÃO
    
    public function getNiveis(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
       
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameNiveisGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getNivelQuestaoAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRQUESTAO', 'P_NRNIVELQUESTAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameNAQGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setNivelQuestAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRNIVELQUESTAVA', $row)) {
                    if ($row['__is_new']) {
                        $row['NRNIVELQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELQUESTAVAL', 1, 22, true); 
                    }
                } else {
                    $row['NRNIVELQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'NIVELQUESTAVAL', 1, 22, true); 
                }
            
            }
            $this->dataSourceName = $this->dataSourceNameNiveis;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteNivelQuestAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameNiveis;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }


    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 