<?php

namespace HCM\FPA\Registro;

class RegistroCNAB extends FPARegistro{

    const CPF = '1';
    const CNPJ = '2';
    const REGISTRO_3 = '3';
    const CNAB_SALARIO_MENSAL = 1;
    // Utilizado em todos os registros de lote de servico.
    const LOTE_SERVICO = '0001';
    // Registros unicos por arquivo.
    const LISTA_TIPO_REGISTROS_UNICO = '0, 1, 5, 9';
    // Registros de lote.
    const LISTA_TIPO_REGISTROS_LOTE = '1, 3, 5';
    // Registro que representa um lote do arquivo (cabecalho).
    const LISTA_TIPO_REGISTRO_CABECALHO_LOTE = '1';
    // Registros que compoe o arquivo.
    const LISTA_TIPO_REGISTROS_ARQUIVO = '0, 1, 3, 5, 9';

    private $_competencia;

    /**
     * Gera uma chave - Parte comum aos diversos registros.
     *
     * @param string $tpRegistro  Numero do Tipo de Registro.
     * @param string $cdBanco     Codigo do Banco.
     * @param string $loteServico Numero do Lote de Servico.
     *
     * @return void
     * @see    FPARegistro::geraRegistro().
     *
     * @author thalesvalias - 13/08/2012
     */
    protected function geraRegistro($tpRegistro, $cdBanco, $loteServico){
        // Bloco 1 | 001-003 | Codigo do Banco.
        $this->atribuiDado($cdBanco, 3);

        // Bloco 2 | 004-007 | Lote do Servico.
        // Numero sequencial. Itera para cada Lote (para cada forma de pagamento. So utilizamos credito em conta)
        // 0000 para header de arquivo, 9999 para trailer de arquivo e 0001 para os demais.
        $this->atribuiDado($loteServico, 4);

        // Bloco 3 | 008 | Tipo do Registro.
        $this->atribuiDado($tpRegistro, 1);
    }


    protected function retornaContaCorrenteSemDigito($cdContaCorrente) {
        $tamanhoContaCorrente = strlen($cdContaCorrente);
        $tamanhoContaCorrenteSemDigito = $tamanhoContaCorrente - 1;
        $cdContaCorrente = substr($cdContaCorrente, 0, $tamanhoContaCorrenteSemDigito);
        return $cdContaCorrente;
    }


    protected function retornaDigitoContaCorrente($cdContaCorrente) {
        $cdContaCorrente = substr($cdContaCorrente, -1);
        return $cdContaCorrente;
    }


    protected function retornaCepSemComplemento($cep) {
        $tamanhoCep = strlen($cep);
        $tamanhoCepSemDigito = $tamanhoCep - 3;
        $cep = substr($cep, 0, $tamanhoCepSemDigito);
        return $cep;
    }


    protected function retornaComplementoCep($cep) {
        $cep = substr($cep, -3);
        return $cep;
    }

    /**
     * Atribui competencia do CNAB.
     *
     * @param TkDateTime $competencia Competencia (data) do CNAB.
     *
     * @return void
     *
     * @author thalesvalias 27/07/2012
     */
    public function setCompetencia($competencia){
        $this->_competencia = $competencia->format('m/Y');
    }


    public function getCompetencia()
    {
        return $this->_competencia;
    }

}