<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 01/03/2016
 * Time: 11:14
 */

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\Geral\Repository\EstruturasRepository;

class EstruturasService {

    protected $entityManager;
    /** @var Environment */
    protected $environment;
    protected $hcmParameters;

    public function __construct(EntityManager $entityManager, Environment $environment, HCMParameters $hcmParameters, EstruturasRepository $estruturasRepository) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcmParameters = $hcmParameters;
        $this->estruturasRepository = $estruturasRepository;
        $this->hierarquiaOrg = Repositories::HIERARQUIAORG;
        $this->estruturam = Repositories::ESTRUTURAM;
        $this->estruturah = Repositories::ESTRUTURAH;
    }
    
    public function deleteEstrutura($dataset) {
        try {
            $this->entityManager->beginTransaction();
            $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(array("nrestruturam" => $dataset['NRESTRUTURAM'], "nrestruturah" => $dataset['NRESTRUTURAH'],"nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($estruturah);
            $this->entityManager->flush();
            $estruturahh = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(array("nrestruturam" => $dataset['NRESTRUTURAM'],"nrorg" => $dataset['NRORG']));
            if(empty($estruturahh)) {
                $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrestruturam" => $dataset['NRESTRUTURAM'],"nrorg" => $dataset['NRORG']));
                $this->entityManager->remove($estruturam);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function getEstruturaSuperior($parameters, $conditions){
        return $this->estruturasRepository->getEstruturaSuperior($parameters, $conditions);
    }
    
    public function getEstruturaLegalGerencial($conditions){
        
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        return $this->estruturasRepository->getEstruturaLegalGerencial($nrorg, $dtmescompetenc, $conditions);
    }
    
    public function getEstruturas($parameters, $conditions) {
        $tipoestrutura = '';
        if(isset($parameters['NRTIPOESTRUTURA'])){
            $nrtipoestrutura = $parameters['NRTIPOESTRUTURA'];
        }    
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        if(!empty($nrtipoestrutura)){
            $tipoestrutura = "AND ESTRUTURAM.NRTIPOESTRUTURA IN ($nrtipoestrutura)";
        }
        $query = $this->entityManager->getConnection()->prepare("
            SELECT * FROM
            (SELECT         
             ESTRUTURAH.*,TIPOESTRUTURA.NMTIPOESTRUTURA,
             ESTRUTURAM.NRTIPOESTRUTURA, ESTRUTURAM.DTINIVIGENCIA, ESTRUTURAM.DTFIMVIGENCIA, ESTRUTURAM.CDINTESTRUTURA, ESTRUTURAM.NMESTRUTURAM,
             ESTRUTURAM.CDINTESTRUTEXP, ESTRUTURAM.CDINTESTRUTEXPFIL, ESTRUTURAM.CDCENTCUST, ESTRUTURAM.NRTPEVENTOCTBL, ESTRUTURAM.NRREGIMEAPUR
              FROM ESTRUTURAM
              LEFT JOIN ESTRUTURAH
                ON ESTRUTURAM.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
               AND ESTRUTURAM.NRORG        = ESTRUTURAH.NRORG
            JOIN TIPOESTRUTURA  ON ESTRUTURAM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA 
             WHERE ESTRUTURAH.NRORG   =  :P_NRORG
               AND ESTRUTURAM.IDATIVO = 'S'
               $tipoestrutura
               AND DTMESCOMPETENC = (SELECT MAX(EH.DTMESCOMPETENC)
                                       FROM ESTRUTURAH EH
                                      WHERE EH.NRORG  =  :P_NRORG
                                        AND EH.NRESTRUTURAM   = ESTRUTURAH.NRESTRUTURAM
                                        AND EH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
               AND (DTFIMVIGENCIA >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR DTFIMVIGENCIA IS NULL)
               
               ORDER BY NMTIPOESTRUTURA, NMESTRUTURAH)
            WHERE 1=1 
            $conditions
        ");
        

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_DTMESCOMPETENC"  => $dtmescompetenc
            )
        );
        
        $result = $query->fetchAll();
        return $result;
    }
    
    public function getEntidadeEducacional($conditions, $filters){
        try{
            $params = array('P_NRESTRUTSUPERIOR' => $conditions['NRESTRUTSUPERIOR'],
                            'P_NRORG'            => $conditions['NRORG'],
                            'P_NRORG_PADRAO'    => $this->hcmParameters->getNrorgPadrao(),
                            'P_DTMESCOMPETENC'   => $conditions['DTMESCOMPETENC']);
            
            return $this->estruturasRepository->getEntidadeEducacional($params, $filters);
        }catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function getEstruturasEducacionaisAdicionar($nrorg, $mesCompetencia, $nrtipoestrutura, $nrhierarquiaorg, $nrestruturasup, $filters){
        try{
            
            return $this->estruturasRepository->getEstruturasEducacionaisAdicionar($nrorg, $mesCompetencia, $nrtipoestrutura, $nrhierarquiaorg, $nrestruturasup, $filters);
        }catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function getLocalidadeEstrutura($parameters) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrestruturam = $parameters["P_NRESTRUTURAM"];
        
        /** @var $parcnegocio \Geral\Entities\Estruturam */
        $parcnegocio = \Geral\Entities\Estruturam::findOneByNrorgAndNrestruturam($nrorg, $nrestruturam);

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRENDERECOPARC,
              INITCAP(MUNICIPIO.NMMUNICIPIO) || '/' || ENDERECOPARC.SGESTADO AS LOCALIDADE
              FROM ENDERECOPARC
              JOIN MUNICIPIO
                ON MUNICIPIO.CDMUNICIPIO = ENDERECOPARC.CDMUNICIPIO
             WHERE ENDERECOPARC.NRORG         = :P_NRORG
               AND ENDERECOPARC.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
        ");

        $query->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_NRPARCNEGOCIO" => $parcnegocio->getNrparcnegocio()
            )
        );

        $result = $query->fetchAll();
        return $result;
    }
    
    public function setEntidadeEducacional($row){
        
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $operador = $this->hcmParameters->getCdoperador();
            $dataAtual = $this->hcmParameters->getDataAtual();
            $mesCompetencia = $this->hcmParameters->getDtCompetencia();
            $nrhierarquia = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'HIERARQUIAORG', 1, 12, true);    
            $tipohierarquia = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy(array('nrtipoestrutsup' => $row['NRTIPOESTRUTLEGAL'], 'nrtipoestrutrel' => 32));
            
            $obj = new $this->hierarquiaOrg;
            
            $obj->setCdoperinclusao($operador);
            $obj->setCdoperultatu($operador);
            $obj->setDtfimvigencia(NULL);
            $obj->setDtinclusao(DateUtil::getDataDeString($dataAtual));
            $obj->setDtinivigencia(DateUtil::getDataDeString($dataAtual));
            $obj->setDtultatu(DateUtil::getDataDeString($dataAtual));
            $obj->setIdativo('S');
            $obj->setNmhierarquiaorg(substr($row['NMESTRUTURAM'], 0, 50));
            $obj->setNrestruturamrel($row['NRESTRUTURAM']);
            $obj->setNrestruturamsup($row['NRESTRUTSUPERIOR']);
            $obj->setNrhierarquiaorg($nrhierarquia);
            $obj->setNrorg($nrorg);
            $obj->setNrorginclusao($nrorg);
            $obj->setNrorgultatu($nrorg);
            $obj->setNrtipohierarqorg($tipohierarquia->getNrtipohierarqorg());
            $this->entityManager->persist($obj);
            $this->entityManager->flush();
        }catch (\Exception $e) {            
            throw $e;
        }
    }
    
    public function novo($nrtipoestrutura, $dtinivigencia, $idativo, $idoptsimples, $nrparcnegocio, $nmestruturam = null, $cdcnpjestrut = null, $dtfimvigencia = null, $cdintestrutura = null, $nrestruturamold = null, $dsestrutura = null, $cdintestrutexp = null,
                         $cdintestrutexpfil = null, $cdcentcust = null, $dsurllogoestrutura = null, $nrtpeventoctbl = null, $nrregimeapur = null, $dtesocial = null, $nrgrupoesocial = null, $dtextincao = null, $nrcalendario = null, $cdceiestrut = null, $nmrazsocestrut = null,
                         $cdnatujuri = null, $cdcnae = null, $nrcaged = null, $cdemprcaixa = null, $idtipoempr = null, $idparticipat = null, $nrpropriempr = null, $nrparcrespinf = null, $nrparcentisin = null, $cdfpas = null, $cdreducont = null, $nmfantasia = null, 
                         $nmestruturah = null, $cdcpfestrutura = null, $dtbasesindical = null, $cdsindical = null, $cdterceiro = null, $nrclasstribut = null, $vrfap = null, $nrtipotribut = null, $vraliqdeson = null, $cdfederalestrut = null, $nrparcregamb = null, 
                         $nrparcmonbio = null, $dtiniresamb = null, $dtfimresamb = null, $nrtpobrigaprendiz = null, $nrtpobrigapcd = null, $cdcaepfestrut = null, $codconvenio = null){
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        
        $estruturam = $this->novoMestre($nrorg, $idativo, $nrtipoestrutura, $dtinivigencia, $nrparcnegocio, $nmestruturam, $dtfimvigencia, $cdintestrutura, $nrestruturamold, $dsestrutura, $cdintestrutexp,
                                        $cdintestrutexpfil, $cdcentcust, $dsurllogoestrutura, $nrtpeventoctbl, $nrregimeapur, $dtesocial, $nrgrupoesocial);
        $estruturah = $this->novoHistorico($nrorg, $idativo, $estruturam->getNrestruturam(), $mesCompetencia, $idoptsimples, $nrparcnegocio, $cdcnpjestrut, $dtextincao, $nrcalendario, $cdceiestrut, $nmrazsocestrut,
                                           $cdnatujuri, $cdcnae, $nrcaged, $cdemprcaixa, $idtipoempr, $idparticipat, $nrpropriempr, $nrparcrespinf, $nrparcentisin, $cdfpas, $cdreducont, $nmfantasia, 
                                           $nmestruturah, $cdcpfestrutura, $dtbasesindical, $cdsindical, $cdterceiro, $nrclasstribut, $vrfap, $nrtipotribut, $vraliqdeson, $cdfederalestrut, $nrparcregamb, 
                                           $nrparcmonbio, $dtiniresamb, $dtfimresamb, $nrtpobrigaprendiz, $nrtpobrigapcd, $cdcaepfestrut, $codconvenio);
        
        return array($estruturam, $estruturah);
        
    }
    
    public function novoMestre($nrorg, $idativo, $nrtipoestrutura, $dtinivigencia, $nrparcnegocio, $nmestruturam, $dtfimvigencia = null, $cdintestrutura = null, $nrestruturamold = null, $dsestrutura = null, $cdintestrutexp = null,
                                $cdintestrutexpfil = null, $cdcentcust = null, $dsurllogoestrutura = null, $nrtpeventoctbl = null, $nrregimeapur = null, $dtesocial = null, $nrgrupoesocial = null){
                                    
        
        $estruturam = new $this->estruturam;
        
        $estruturam->setNrorg($nrorg);
        $estruturam->setNrtipoestrutura($nrtipoestrutura);
        $estruturam->setDtinivigencia(DateUtil::getDataDeString($dtinivigencia));
        $estruturam->setNrparcnegocio($nrparcnegocio);
        $estruturam->setNmestruturam($nmestruturam);
        $estruturam->setIdativo($idativo);
        
        $estruturam->setDtfimvigencia(DateUtil::getDataDeString($dtfimvigencia));
        $estruturam->setCdintestrutura($cdintestrutura);
        $estruturam->setNrestruturamold($nrestruturamold);
        // $estruturam->setDsestrutura($dsestrutura);
        // $estruturam->setCdintestrutexp($cdintestrutexp);
        // $estruturam->setCdintestrutexpfil($cdintestrutexpfil);
        // $estruturam->setCdcentcust($cdcentcust);
        // $estruturam->setDsurllogoestrutura($dsurllogoestrutura);
        $estruturam->setNrtpeventoctbl($nrtpeventoctbl);
        // $estruturam->setNrregimeapur($nrregimeapur);
        // $estruturam->setDtesocial($dtesocial);
        // $estruturam->setNrgrupoesocial($nrgrupoesocial);       
        

        $estruturam->setDtinclusao(DateUtil::getDataAtual());
        $estruturam->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $estruturam->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $estruturam->setDtultatu(DateUtil::getDataAtual());
        $estruturam->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $estruturam->setCdoperultatu($this->hcmParameters->getCdoperador());

        $estruturam->setNrestruturam(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESTRUTURAM', 1, 12, true));

        $this->entityManager->persist($estruturam);
        $this->entityManager->flush($estruturam);
    
        return $estruturam;
        
    }
    
    public function novoHistorico($nrorg, $idativo, $nrestruturam, $mesCompetencia, $idoptsimples, $nrparcnegocio, $cdcnpjestrut = null, $dtextincao = null, $nrcalendario = null, $cdceiestrut = null, $nmrazsocestrut = null,
                                  $cdnatujuri = null, $cdcnae = null, $nrcaged = null, $cdemprcaixa = null, $idtipoempr = null, $idparticipat = null, $nrpropriempr = null, $nrparcrespinf = null, $nrparcentisin = null, $cdfpas = null, $cdreducont = null, $nmfantasia = null, 
                                  $nmestruturah = null, $cdcpfestrutura = null, $dtbasesindical = null, $cdsindical = null, $cdterceiro = null, $nrclasstribut = null, $vrfap = null, $nrtipotribut = null, $vraliqdeson = null, $cdfederalestrut = null, $nrparcregamb = null, 
                                  $nrparcmonbio = null, $dtiniresamb = null, $dtfimresamb = null, $nrtpobrigaprendiz = null, $nrtpobrigapcd = null, $cdcaepfestrut = null, $codconvenio = null){
        
        $estruturah = new $this->estruturah;
        
        $estruturah->setNrorg($nrorg);
        $estruturah->setNrestruturam($nrestruturam);
        $estruturah->setDtmescompetenc(DateUtil::getDataDeString($mesCompetencia));
        $estruturah->setCdcnpjestrut($cdcnpjestrut);
        $estruturah->setIdoptsimples($idoptsimples);
        
        $estruturah->setDtextincao(DateUtil::getDataDeString($dtextincao));
        $estruturah->setNrcalendario($nrcalendario);
        $estruturah->setCdceiestrut($cdceiestrut);
        $estruturah->setNmrazsocestrut($nmrazsocestrut);
        $estruturah->setCdnatujuri($cdnatujuri);
        $estruturah->setCdcnae($cdcnae);
        $estruturah->setNrcaged($nrcaged);
        $estruturah->setCdemprcaixa($cdemprcaixa);
        $estruturah->setIdtipoempr($idtipoempr);
        $estruturah->setIdparticipat($idparticipat);
        $estruturah->setNrparcnegocio($nrparcnegocio);
        $estruturah->setNrpropriempr($nrpropriempr);
        $estruturah->setNrparcrespinf($nrparcrespinf);
        $estruturah->setNrparcentisin($nrparcentisin);
        $estruturah->setCdfpas($cdfpas);
        $estruturah->setCdreducont($cdreducont);
        $estruturah->setNmfantasia($nmfantasia);
        $estruturah->setNmestruturah($nmestruturah);
        $estruturah->setCdcpfestrutura($cdcpfestrutura);
        $estruturah->setDtbasesindical($dtbasesindical);
        $estruturah->setCdsindical($cdsindical);
        $estruturah->setCdterceiro($cdterceiro);
        $estruturah->setNrclasstribut($nrclasstribut);
        $estruturah->setVrfap($vrfap);
        $estruturah->setNrtipotribut($nrtipotribut);
        $estruturah->setVraliqdeson($vraliqdeson);
        $estruturah->setCdfederalestrut($cdfederalestrut);
        $estruturah->setIdativo($idativo);
        $estruturah->setNrparcregamb($nrparcregamb);
        $estruturah->setNrparcmonbio($nrparcmonbio);
        $estruturah->setDtiniresamb($dtiniresamb);
        $estruturah->setDtfimresamb($dtfimresamb);
        // $estruturah->setNrtpobrigaprendiz($nrtpobrigaprendiz);
        // $estruturah->setNrtpobrigapcd($nrtpobrigapcd);
        // $estruturah->setCdcaepfestrut($cdcaepfestrut);
        // $estruturah->setCodconvenio($codconvenio);
        
        $estruturah->setDtinclusao(DateUtil::getDataAtual());
        $estruturah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $estruturah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $estruturah->setDtultatu(DateUtil::getDataAtual());
        $estruturah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $estruturah->setCdoperultatu($this->hcmParameters->getCdoperador());

        $estruturah->setNrestruturam(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESTRUTURAH', 1, 12, true));

        $this->entityManager->persist($estruturah);
        $this->entityManager->flush($estruturah);
        
        return $estruturah;
        
    }
    
    public function deleteEntidadeEducacional($nrorg, $nrhierarquiaorg){
        try{    
            $this->entityManager->beginTransaction();
            $hierarquia = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findOneBy(array("nrorg" => $nrorg, "nrhierarquiaorg" => $nrhierarquiaorg));
            $this->entityManager->remove($hierarquia);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }catch (\Exception $e) {            
            throw $e;
        }
    } 
    
    public function retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc){    
        return $this->estruturasRepository->retornaDadosResponsavelInformal($nrorg, $nrestrutgeren, $dtmescompetenc);
    }    
    
    public function retornaEstrutura($nrEstrutura){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $estrutura = $this->estruturasRepository->retornaEstrutura($nrEstrutura, $nrorg, $dtmescompetenc);
        
        if (is_object($estrutura)) {
            // Retorna o historico mais atual, de acordo com a data.
            $historico = $this->estruturasRepository->retornaEstruturaHistorico($nrEstrutura, $nrorg, $dtmescompetenc);
            $estrutura->historico = $historico;
            return $estrutura;
        } else {
            return null;
        }
    }
}