<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query;
use Doctrine\ORM\QueryBuilder;
use Setforms2\Shared\ORM\DateTime;
use Zeedhi\Framework\HTTP\Kernel;
use Teknisa\Libs\Util\Environment;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\DateUtil;
use HCM\Util\NovoCodigo;
use Gestaopessoas\Entities\PtoPeriodoapuracao;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Gestaopessoas\Entity\GpeAlteescala;
use HCM\Gestaopessoas\Entity\GpeIntervaloshorario;
use HCM\Gestaopessoas\Entity\GpeMarcacaoponto;
use HCM\Gestaopessoas\Entity\GpeVinculoh;
use HCM\Gestaopessoas\Entity\PtoProgocorrencia;
use HCM\Util\HCMParameters;
use HCM\Geral\Repository\AlteEscalaRepository;
use HCM\PTO\Repository\HorarioRepository;

class MarcacaoPontoService {

    protected $entityManager;
    /** @var Environment */
    protected $environment;
    
    private const HR_INICIO_VALIDACAO_MARC_DA   = '00:00:00';
    private const HR_FIM_VALIDACAO_MARC_DA      = '07:00:00';

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Environment $environment, Kernel $httpKernel, AlteEscalaRepository $alteEscalaRepository, HorarioRepository $horarioRepository) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->httpKernel = $httpKernel;
        $this->hcmParameters = $hcmParameters;
        $this->alteEscalaRepository = $alteEscalaRepository;
        $this->horarioRepository = $horarioRepository;
        $this->gpeVinculomEntity = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->gpeMarcacaoponto = Repositories::GPE_MARCACAOPONTO;
        $this->ptoProgOcorrencia = Repositories::PTO_PROGOCORRENCIA;
    }
    
    public function getMarcacoesPonto($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT 
                TO_CHAR(S.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS SAIDA,
                TO_CHAR(E.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS ENTRADA,
                S.NRMARCACAOPONTO AS NRMARCACAOPONTO,
                S.IDENTRADASAIDA,
                TO_CHAR(S.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') || ' - ' || TO_CHAR(E.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS DESCRIPTION,
                S.NRORG
            FROM
                (SELECT MCP.DTHRAJUSTADA,
                    MCP.NRMARCACAOPONTO,
                    MCP.NRORG,
                    CASE WHEN MOD(ROWNUM, 2) = 0 THEN 'S'
                    ELSE 'E'
                    END AS IDENTRADASAIDA,
                    ROW_NUMBER() OVER(PARTITION BY NRORG ORDER BY DTHRAJUSTADA) AS NRORDER_INTERV FROM(SELECT MP.*
                        FROM GPE_MARCACAOPONTO MP WHERE MP.NRVINCULOM = :P_NRVINCULOM
                        AND MP.NRORG = :NRORG
                        AND MP.NRCOLETOR NOT IN(
                            (SELECT PORG.NRCOLETORMARCAUTOM FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCGERSIST FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCPROJETADA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCVIRADIA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG))) MCP) S,
                (SELECT MCP.*,
                    CASE WHEN MOD(ROWNUM, 2) = 0 THEN 'S'
                    ELSE 'E'
                    END AS IDENTRADASAIDA, ((ROW_NUMBER() OVER(PARTITION BY NRORG ORDER BY DTHRAJUSTADA)) - 1) AS NRORDER_INTERV FROM(SELECT MP.*
                        FROM GPE_MARCACAOPONTO MP WHERE MP.NRVINCULOM = :P_NRVINCULOM
                        AND MP.NRORG = :NRORG
                        AND MP.NRCOLETOR NOT IN(
                            (SELECT PORG.NRCOLETORMARCAUTOM FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCGERSIST FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCPROJETADA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCVIRADIA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG))) MCP) E
            WHERE S.NRORG = E.NRORG
            AND S.IDENTRADASAIDA = 'S'
            AND S.NRORDER_INTERV = E.NRORDER_INTERV
            AND(((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA IN('DA',
                'DC',
                'DS')) AND TO_DATE(TO_CHAR(E.DTHRAJUSTADA, 'DD/MM/YYYY')) = :P_DTAPURACAOPONTO) OR(IDDIAOCORRENCIA = 'FA'
                AND TRUNC(S.DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') + 1) OR(IDDIAOCORRENCIA = 'FS'
                AND TRUNC(S.DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') - 1) OR(IDDIAOCORRENCIA = 'FC'
                AND TRUNC(S.DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')))
            AND(TRUNC(E.DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') OR((IDDIAOCORRENCIA = 'FA'
                OR IDDIAOCORRENCIA = 'DA') AND TRUNC(E.DTHRAJUSTADA) = (TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY') + 1)))
        ");
        $query->execute(
            array(
                "NRORG"           => $parameters["P_NRORG"],
                "P_NRVINCULOM"      => $parameters["P_NRVINCULOM"],
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );
        $marcacaoPontos = $query->fetchAll();
               
          if (isset($marcacaoPontos[0])) {
            return $marcacaoPontos[0];
        } else {
            return $marcacaoPontos;
        }

    }
    
    public function getMarcacaoPonto($parameters) {
//        $parameters["P_NRVINCULOM"] = '17196';

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              ROWNUM,
              NRMARCACAOPONTO,
              NRVINCULOM,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS DTHRAJUSTADA,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
              DTHRAJUSTADA AS DTHRORDER,
              IDMARCDESPREZADA,
              IDDIAOCORRENCIA,
              DSJUSTIFICATIVA,
              COL.NRCOLETOR,
              COL.DSCOLETOR
             FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
               ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND NRVINCULOM = :P_NRVINCULOM
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA IN ('DA', 'DC', 'DS'))
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                   OR (IDDIAOCORRENCIA = 'FA' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA = 'FS' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
              /*AND (TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')
               OR ((IDDIAOCORRENCIA    = 'FA'
                    OR IDDIAOCORRENCIA = 'DA')
                  AND TRUNC(DTHRAJUSTADA) = (TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)))*/
            ORDER BY DTHRORDER, NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $parameters["P_NRORG"],
                "P_NRVINCULOM"      => $parameters["P_NRVINCULOM"],
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );
    
        $marcacoesPonto = $query->fetchAll();
        $intervalos = $this->factoryIntervaloMarcacoes($marcacoesPonto);

        if (isset($intervalos)) {
            return $intervalos;
        } else {
            return null;
        }

    }
    
    public function factoryIntervaloMarcacoes($marcacoesPonto) {

        $intervalosHorario = array();

        // gera os intervalos de entrada e calcula o tempo
        $intervalosEntrada = $this->getIntervalosHorario($marcacoesPonto, 'Trabalho');

        // remove as marcações de primeira entrada (chegada) e última sáida (embora)
        unset($marcacoesPonto[0]);
        array_pop($marcacoesPonto);

        // gera os intervalos de saída e calcula o tempo
        $intervalosSaida = $this->getIntervalosHorario($marcacoesPonto, 'Ausência');

        // uni os arrays de intervalo de entrada e saída
        $intervalosHorario = array_merge($intervalosEntrada, $intervalosSaida);

        // reordena o array pelo o horário, colocando-o como chave para facilitar a ordenação.
        $sortIntervalosHorario = array();
        for ($i = 0; $i < count($intervalosHorario); $i++) {
            $key = str_replace(":", "", $intervalosHorario[$i]["DTHRINICIO"].$i);
            $sortIntervalosHorario[$key] = $intervalosHorario[$i];
        }
        ksort($sortIntervalosHorario);

        $returnIntervalosHorario = array();
        foreach ($sortIntervalosHorario as $value) {
            $returnIntervalosHorario[] = $value;
        }

        return $returnIntervalosHorario;

    }
    
     private function getIntervalosHorario($marcacoesPonto, $idmarcacao) {

        $paresMarcacoes = array_chunk($marcacoesPonto, 2);

        $intervalos = array();
        foreach ($paresMarcacoes as $key => $intervalosMarcacoes) {
            /* Posição 0 do array "$intervalosMarcacoes" são marcações de início
             * Posição 1 do array "$intervalosMarcacoes" são marcações de fim
             */

            $intervalos[$key]["NRMARCACAOPONTO"]       = $intervalosMarcacoes[0]["NRMARCACAOPONTO"];
            $intervalos[$key]["NRMARCACAOPONTO_SAIDA"] = $intervalosMarcacoes[1]["NRMARCACAOPONTO"];

            $intervalos[$key]["IDMARCACAO"] = $idmarcacao;

            $intervalos[$key]["DTHRINICIO"] = $intervalosMarcacoes[0]["HORA_MINUTO"];
            $intervalos[$key]["DTHRFIM"]    = $intervalosMarcacoes[1]["HORA_MINUTO"];

            $horaEntrada = $intervalosMarcacoes[0]["HORA_MINUTO"];
            $horaSaida   = $intervalosMarcacoes[1]["HORA_MINUTO"];
            $tempoIntervalo = DateUtil::subtrairHoras($horaSaida, $horaEntrada);
            $intervalos[$key]["TEMPO"]      = $tempoIntervalo;

            $identificaIntervalo = $this->verificaIdentificaIntervalo($intervalosMarcacoes[0]);
            if (isset($identificaIntervalo["NRPROGOCORRENCIA"])) {
                $intervalos[$key]["ISALMOCO"] = "true";
                $intervalos[$key]["NRPROGOCORRENCIA_ALMOCO"] = $identificaIntervalo["NRPROGOCORRENCIA"];
            } else {
                $intervalos[$key]["ISALMOCO"] = "false";
                $intervalos[$key]["NRPROGOCORRENCIA_ALMOCO"] = '';
            }

            $abonarSaida = $this->verificaAbonarSaida($intervalosMarcacoes[0]);
            if (isset($abonarSaida["NRPROGOCORRENCIA"])) {
                $intervalos[$key]["ISJUSTIFICADA"] = "true";
                $intervalos[$key]["NRPROGOCORRENCIA_JUSTIFICADA"]          = $abonarSaida["NRPROGOCORRENCIA"];
                $intervalos[$key]["NRMOTIVOOCORFREQ_LABEL"]                = $abonarSaida["DSMOTIVOOCORFREQ"];
                $intervalos[$key]["NRMOTIVOOCORFREQ"]                      = $abonarSaida["NRMOTIVOOCORFREQ"];
                $intervalos[$key]["NRPROGOCORRENCIA_DSJUSTIFICATIVA"]      = $abonarSaida["DSOBSERVACAO"];
                $intervalos[$key]["NRPROGOCORRENCIA_IDPROGOCORCONFIRMADA"] = $abonarSaida["IDPROGOCORCONFIRMADA"];
            } else {
                $intervalos[$key]["ISJUSTIFICADA"] = "false";
                $intervalos[$key]["NRPROGOCORRENCIA_JUSTIFICADA"]          = '';
                $intervalos[$key]["NRMOTIVOOCORFREQ_LABEL"]                = '';
                $intervalos[$key]["NRMOTIVOOCORFREQ"]                      = '';
                $intervalos[$key]["NRPROGOCORRENCIA_DSJUSTIFICATIVA"]      = '';
                $intervalos[$key]["NRPROGOCORRENCIA_IDPROGOCORCONFIRMADA"] = '';
            }

        }

        return $intervalos;

    }

    private function verificaIdentificaIntervalo($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT PROG.*
              FROM PTO_PROGOCORRENCIA PROG
              JOIN GPE_MARCACAOPONTO MARC
                ON MARC.NRMARCACAOPONTO = PROG.NRMARCACAOPONTO
               AND MARC.NRVINCULOM      = PROG.NRVINCULOM
               AND MARC.NRORG           = PROG.NRORG
             WHERE PROG.NRORG                 = :P_NRORG
               AND PROG.NRVINCULOM            = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA IN (20, 26)
               AND PROG.DTINICIOPROGOCOR      = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO       = :P_NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $this->environment->getNrOrgTrab(),
                "P_NRVINCULOM"      => $parameters["NRVINCULOM"],
                "P_DTMARCACAOPONTO" => $parameters["DTMARCACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["NRMARCACAOPONTO"]
            )
        );

        $ocorrencias = $query->fetchAll();

        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function verificaAbonarSaida($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT
                *
              FROM PTO_PROGOCORRENCIA PROG
              LEFT JOIN PTO_MOTIVOOCORFREQ MOT
                ON MOT.NRMOTIVOOCORFREQ = PROG.NRMOTIVOOCORFREQ
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA = 17
               AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.DTFIMPROGOCOR        = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO      = :P_NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $this->environment->getNrOrgTrab(),
                "P_NRVINCULOM"      => $parameters["NRVINCULOM"],
                "P_DTMARCACAOPONTO" => $parameters["DTMARCACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["NRMARCACAOPONTO"]
            )
        );

        $ocorrencias = $query->fetchAll();

        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    public function getDadosHorarioVinculo($nrorg, $nrvinculom, $dtapuracaoponto, $dtmescompetenc = null) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT HT.NRHORDIAM, HD.IDTIPOHORARIO, IH.NRINTERVALOHORARIO, HH.NMHORDIAH, AE.NRESCALATRABM, EH.QTHRESCALATRABH AS QTJORNADADIARIA
		  FROM (SELECT NRORG, MAX(NRTURNO) NRTURNO, NRESCALATRABM
					 FROM GPE_ALTEESCALA
					WHERE NRORG = :P_NRORG
					  AND NRVINCULOM = :P_NRVINCULOM
					  --AND TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
                      AND DTINIESCALA = (SELECT MAX(DTINIESCALA)
                                 FROM GPE_ALTEESCALA ALT
                                WHERE ALT.NRORG        = GPE_ALTEESCALA.NRORG
                                  AND ALT.NRVINCULOM   = GPE_ALTEESCALA.NRVINCULOM
                                  AND ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
					GROUP BY NRORG, NRESCALATRABM) AE,
				 (SELECT NRORG, NRTURNO, TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')-DTDATABASE AS NRDIASENTRE
					 FROM GPE_TURNO
					WHERE NRORG = :P_NRORG) TU,
				 (SELECT NRORG, NRTURNO, NRHORDIAM,
							ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					ORDER BY NRORG, NRTURNO, NRSEQHORARIO) HT,
				 (SELECT NRORG, NRTURNO, COUNT(*) AS QTHORARIOS
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRTURNO) QH,
				 (SELECT NRORG, NRHORDIAM, MAX(DTMESCOMPETENC) AS DTMESCOMPETENC, NMHORDIAH
					 FROM GPE_HORDIAH
					WHERE NRORG = :P_NRORG
					  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
					GROUP BY NRORG, NRHORDIAM, NMHORDIAH) HH,
				 (SELECT NRORG, NRHORDIAM, MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO
					 FROM GPE_INTERVALOSHORARIO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRHORDIAM) IV, GPE_INTERVALOSHORARIO IH, GPE_HORDIAH HD, GPE_ESCALATRABH EH
		 WHERE AE.NRORG = TU.NRORG
			AND AE.NRTURNO = TU.NRTURNO
			AND AE.NRORG = HT.NRORG
			AND AE.NRTURNO = HT.NRTURNO
			AND HT.NRORG = HH.NRORG
			AND HT.NRHORDIAM = HH.NRHORDIAM
			AND HD.NRORG = HH.NRORG
			AND HD.NRHORDIAM = HH.NRHORDIAM
			AND HD.DTMESCOMPETENC = HH.DTMESCOMPETENC
			AND IV.NRORG(+) = HH.NRORG
			AND IV.NRHORDIAM(+) = HH.NRHORDIAM
			AND IH.NRORG(+) = IV.NRORG
			AND IH.NRHORDIAM(+) = IV.NRHORDIAM
			AND IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO
			AND AE.NRORG = QH.NRORG
			AND AE.NRTURNO = QH.NRTURNO
			AND HT.NRSEQHR-1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS)
      AND EH.NRESCALATRABM = AE.NRESCALATRABM
      AND EH.NRORG = AE.NRORG
      AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                 FROM GPE_ESCALATRABH
                                WHERE NRORG = EH.NRORG
                                  AND NRESCALATRABM = AE.NRESCALATRABM
                                  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
        ");

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTMESCOMPETENCIA" => is_null($dtmescompetenc) ? DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO) : $dtmescompetenc,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
            )
        );
        $dadosHorario = $query->fetchAll();

        if (isset($dadosHorario[0])) {
            return $dadosHorario[0];
        } else {
            return false;
        }
    }

    public function deleteMarcacoesPonto($rows) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        foreach($rows as $row){

            $nrmarcacaoponto_entrada = $row['NRMARCACAOPONTO'];
            $nrmarcacaoponto_saida   = $row['NRMARCACAOPONTO_SAIDA'];

            $query = "DELETE FROM
                        PTO_PROGOCORRENCIA
                      WHERE NRORG             = $nrorg
                        AND NRVINCULOM        = $nrvinculom
                        AND NRMARCACAOPONTO  IN ($nrmarcacaoponto_entrada, $nrmarcacaoponto_saida)";

            $result_ocorrencia = $this->entityManager->getConnection()->exec($query);

            $query_marc = "DELETE FROM
                      GPE_MARCACAOPONTO
                    WHERE NRORG      = $nrorg
                      AND NRVINCULOM = $nrvinculom
                      AND NRMARCACAOPONTO IN ($nrmarcacaoponto_entrada, $nrmarcacaoponto_saida)";

            $result_marcacao = $this->entityManager->getConnection()->exec($query_marc);

        }

        //if ($result_marcacao > 0)
            return array('DELETE' => true);
        /*else
            return array('DELETE' => false);*/

    }

    public function ausenciaAntesJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRMARCACAOPONTO,
              TO_CHAR(DTHRMARCACAOPONTO, 'HH24:MI') DTHRMARCACAOPONTO,
              (SELECT
                  SUBSTR(HRENTRADA, 1, 2) || ':' || SUBSTR(HRENTRADA, 3, 2) HRENTRADA
                 FROM GPE_INTERVALOSHORARIO
                WHERE NRORG = :P_NRORG
                  AND NRHORDIAM = :P_NRHORDIAM
                  AND HRENTRADA = (SELECT MIN(HRENTRADA)
                                     FROM GPE_INTERVALOSHORARIO INTER
                                    WHERE INTER.NRORG = GPE_INTERVALOSHORARIO.NRORG
                                      AND INTER.NRHORDIAM = GPE_INTERVALOSHORARIO.NRHORDIAM)) HRBASEENTRADA
             FROM GPE_MARCACAOPONTO
            WHERE NRORG      = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND NRCOLETOR  = 24
              AND DTHRMARCACAOPONTO = (SELECT MIN(MARC.DTHRMARCACAOPONTO)
                                         FROM GPE_MARCACAOPONTO MARC
                                        WHERE MARC.NRORG      = GPE_MARCACAOPONTO.NRORG
                                          AND MARC.NRVINCULOM = GPE_MARCACAOPONTO.NRVINCULOM
                                          AND MARC.NRCOLETOR  = GPE_MARCACAOPONTO.NRCOLETOR
                                          AND MARC.IDDIAOCORRENCIA IN ('FC', 'DC')
                                          AND MARC.DSJUSTIFICATIVA IS NULL
                                          AND TRUNC(MARC.DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRHORDIAM"       => $parameters["P_NRHORDIAM"]
            )
        );

        $ausenciaAntesJornada = $query->fetchAll();

        if (isset($ausenciaAntesJornada[0])) {
            return $ausenciaAntesJornada[0];
        } else {
            return null;
        }
    }

    public function ausenciaDepoisJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              MAX(NRMARCACAOPONTO) NRMARCACAOPONTO,
              TO_CHAR(DTHRMARCACAOPONTO, 'HH24:MI') DTHRMARCACAOPONTO,
              (SELECT
                  SUBSTR(HRSAIDA, 1, 2) || ':' || SUBSTR(HRSAIDA, 3, 2) HRSAIDA
                 FROM GPE_INTERVALOSHORARIO
                WHERE NRORG = :P_NRORG
                  AND NRHORDIAM = :P_NRHORDIAM
                  AND HRSAIDA   = (SELECT MAX(HRSAIDA)
                                     FROM GPE_INTERVALOSHORARIO INTER
                                    WHERE INTER.NRORG = GPE_INTERVALOSHORARIO.NRORG
                                      AND INTER.NRHORDIAM = GPE_INTERVALOSHORARIO.NRHORDIAM)) HRBASESAIDA
             FROM GPE_MARCACAOPONTO
            WHERE NRORG      = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND NRCOLETOR  = 24
              AND DTHRMARCACAOPONTO = (SELECT MAX(MARC.DTHRMARCACAOPONTO)
                                         FROM GPE_MARCACAOPONTO MARC
                                        WHERE MARC.NRORG      = GPE_MARCACAOPONTO.NRORG
                                          AND MARC.NRVINCULOM = GPE_MARCACAOPONTO.NRVINCULOM
                                          AND MARC.NRCOLETOR  = GPE_MARCACAOPONTO.NRCOLETOR
                                          AND MARC.IDDIAOCORRENCIA IN ('FC', 'DC')
                                          AND MARC.DSJUSTIFICATIVA IS NULL
                                          AND TRUNC(MARC.DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
            GROUP BY DTHRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRHORDIAM"       => $parameters["P_NRHORDIAM"]
            )
        );

        $ausenciaDepoisJornada = $query->fetchAll();

        if (isset($ausenciaDepoisJornada[0])) {
            return $ausenciaDepoisJornada[0];
        } else {
            return null;
        }
    }

    public function getNrmarcacaoPontoSaidaAusenciaAntesJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              MAX(NRMARCACAOPONTO) NRMARCACAOPONTO_SAID
             FROM GPE_MARCACAOPONTO
            WHERE NRORG               = :P_NRORG
              AND NRVINCULOM          = :P_NRVINCULOM
              AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND DSJUSTIFICATIVA IS NOT NULL
              AND NRCOLETOR = 24
              AND ROWNUM <= 2
            ORDER BY DTHRMARCACAOPONTO, NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );

        $nrMarcacaoSaidaAntesJornada = $query->fetchAll();

        if (isset($nrMarcacaoSaidaAntesJornada[0])) {
            return $nrMarcacaoSaidaAntesJornada[0];
        } else {
            return null;
        }
    }

    public function setIdfixaentsaidMarcacaoPonto($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND IDFIXAENTSAID            = :P_IDFIXAENTSAID
        ");

        $return = $sql->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_IDFIXAENTSAID"   => $parameters["P_IDFIXAENTSAID"]
            )
        );

        $query = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = :P_IDFIXAENTSAID
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRMARCACAOPONTO          = :P_NRMARCACAOPONTO
        ");

        $result_update = $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["P_NRMARCACAOPONTO"],
                "P_IDFIXAENTSAID"   => $parameters["P_IDFIXAENTSAID"]
            )
        );

        if ($result_update)
            return array('UPDATE' => true);
        else
            return array('UPDATE' => false);
    }

    public function importaDigitacaoAutomatica($parameters) {
        $nrorg           = $this->environment->getNrOrg();
        $nrvinculom      = $this->environment->getNrvinculom();
        $dtapuracaoponto = $parameters["P_DTAPURACAOPONTO"];
        $nrinscricaoparc = $parameters["P_NRINSCRICAOPARC"];

        $connProducaoTeknisa = oci_connect('PRODUCAO_CONSULTA', 'tecnisa', '192.168.122.6/PDBORCL');

        // retorna horário base para validações de 5 minutos na entrada
        $horariobase = $this->getHorarioBase($nrorg, $parameters["P_NRHORDIAM"]);
        $dtHoraBaseEntrada = $dtapuracaoponto. ' ' .$horariobase['HRENTRADA_NOVA'];
        $horaBaseEntrada   = $horariobase['HRENTRADA_NOVA'];

        /*$query = "
            SELECT
                FU.CDFUNC,
                FU.NMFUNC,
                FU.NRCPFFUNC,
                FU.USR_NRVINCULO_SAAS,
                FU.USR_NRORG_SAAS,
                TO_CHAR(DT.DTDATA, 'DD/MM/YYYY')      DTDATA,
                TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI') DTBATIDAPTO
              FROM DATA DT
              JOIN FUNCIONARIO FU
                ON FU.NRORG     = 1
               AND FU.CDEMPRESA = '01'
               AND (FU.USR_NRVINCULO_SAAS IS NOT NULL OR FU.USR_NRVINCULO_SAAS IS NULL)
               AND (FU.USR_NRORG_SAAS     IS NOT NULL OR FU.USR_NRORG_SAAS     IS NULL)
              JOIN BATIDAPONTO PONTO
                ON PONTO.NRORG              = FU.NRORG
               AND PONTO.CDFUNC             = FU.CDFUNC
               AND TRUNC(PONTO.DTBATIDAPTO) = DT.DTDATA
              WHERE DT.DTDATA BETWEEN TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY') AND TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY')
                AND (FU.USR_NRVINCULO_SAAS = '$nrvinculom'
                     OR FU.NRCPFFUNC = '$nrinscricaoparc')
              ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO
        ";*/

        // LEONARDOCARVALHO - 16/02/2016 - Nova query tratando tolerância de 5 minutos na entrada
        $query = "
            SELECT
              RES.CDFUNC,
              RES.NMFUNC,
              RES.NRCPFFUNC,
              RES.USR_NRVINCULO_SAAS,
              RES.DTDATA,
              RES.DTBATIDAPTO_REAL,
              RES.DTBATIDAPTO,
              RES.DTBATIDAPTO_SAIDA
            FROM (SELECT
                    SEG.*,
                    CASE
                      WHEN TO_DATE(SEG.DTBATIDAPTO_REAL, 'HH24:MI') < TO_DATE('$horaBaseEntrada', 'HH24:MI') THEN 'AN'
                      WHEN TO_DATE(SEG.DTBATIDAPTO_REAL, 'HH24:MI') > TO_DATE('$horaBaseEntrada', 'HH24:MI') + 0.003472 THEN 'AP'
                      ELSE 'DE'
                    END FLAG1,
                    CASE
                      WHEN TO_DATE(SEG.DTBATIDAPTO_SAIDA, 'HH24:MI') < TO_DATE('$horaBaseEntrada', 'HH24:MI') THEN 'AN'
                      WHEN TO_DATE(SEG.DTBATIDAPTO_SAIDA, 'HH24:MI') > TO_DATE('$horaBaseEntrada', 'HH24:MI') + 0.003472 THEN 'AP'
                      ELSE 'DE'
                    END FLAG2
                  FROM(SELECT
                          PRI.*,
                          LEAD (PRI.DTBATIDAPTO_REAL, 1) OVER (ORDER BY PRI.USR_NRVINCULO_SAAS, PRI.DTBATIDAPTO) DTBATIDAPTO_SAIDA,
                          ROWNUM LIN
                        FROM (SELECT
                                CEN.*
                              FROM(SELECT
                                     FU.CDFUNC,
                                     FU.NMFUNC,
                                     FU.NRCPFFUNC,
                                     FU.USR_NRVINCULO_SAAS,
                                     FU.USR_NRORG_SAAS,
                                     PONTO.IDSENTBATIDA,
                                     TO_CHAR(DT.DTDATA, 'DD/MM/YYYY')      DTDATA,
                                     TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI') DTBATIDAPTO_REAL,
                                     DECODE(
                                       TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI') - PONTO.DTBATIDAPTO,
                                       ABS(TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI') - PONTO.DTBATIDAPTO),
                                       TO_CHAR(TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI'), 'HH24:MI'),
                                       TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI')) DTBATIDAPTO,
                                     ROW_NUMBER() OVER (ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO) L
                                   FROM DATA DT
                                   JOIN FUNCIONARIO FU
                                     ON FU.NRORG     = 1
                                    AND FU.CDEMPRESA = '01'
                                    AND (FU.USR_NRVINCULO_SAAS IS NOT NULL OR FU.USR_NRVINCULO_SAAS IS NULL)
                                    AND (FU.USR_NRORG_SAAS     IS NOT NULL OR FU.USR_NRORG_SAAS     IS NULL)
                                   JOIN BATIDAPONTO PONTO
                                     ON PONTO.NRORG              = FU.NRORG
                                    AND PONTO.CDFUNC             = FU.CDFUNC
                                    AND TRUNC(PONTO.DTBATIDAPTO) = DT.DTDATA
                                   WHERE DT.DTDATA BETWEEN TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY') AND TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY')
                                     AND (FU.USR_NRVINCULO_SAAS = '$nrvinculom'
                                          OR FU.NRCPFFUNC = '$nrinscricaoparc')
                                   ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO
                                   ) CEN
                                   WHERE CEN.L != 1 OR CEN.IDSENTBATIDA != 2 /* Necessário para ignorar se o primeiro registro for uma saída */
                             ) PRI
                  ) SEG
                  WHERE MOD(SEG.LIN, 2) = 1
            ) RES
            WHERE ( RES.FLAG1 != 'AN' OR RES.FLAG2 != 'DE' ) AND ( RES.FLAG1 != 'AN' OR RES.FLAG2 != 'AN' )
              AND RES.DTBATIDAPTO_SAIDA IS NOT NULL
        ";

        $executeProducaoTeknisa = oci_parse($connProducaoTeknisa, $query);
        oci_execute($executeProducaoTeknisa);
        oci_fetch_all($executeProducaoTeknisa, $marcacoesCatraca, null, null, OCI_FETCHSTATEMENT_BY_ROW);

        $resultEnt = false;
        $resultSai = false;
        if (count($marcacoesCatraca) > 0) {

            // se total de marcações forem impar, retira a última marcação para formar sempre intervalos - NOVA QUERY
            /*if (count($marcacoesCatraca) % 2 != 0) {
                array_pop($marcacoesCatraca);
            }*/

            $dataset = array();
            foreach ($marcacoesCatraca as $marcacaoCatraca) {
                if ($marcacaoCatraca["USR_NRVINCULO_SAAS"] == $nrvinculom ||
                    $marcacaoCatraca["NRCPFFUNC"] == $nrinscricaoparc
                ) {

                    $dataset["idenquadrahorario"] = 'N';
                    $dataset["idmarcdesprezada"] = 'N';
                    $dataset["iddiaocorrencia"] = 'FC';
                    $dataset["dsjustificativa"] = '';
                    $dataset["nrorginclusao"] = '';
                    $dataset["nrcoletor"] = '24';
                    $dataset["idativo"] = 'S';
                    $dataset["dtinclusao"] = DateUtil::getDataAtual();
                    $dataset["cdoperinclusao"] = '';

                    // insere marcação de entrada
                    $dataset["dthrmarcacaoponto"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO"] . ":00";
                    $dataset["dthrajustada"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO"] . ":00";

                    $resultEnt = $this->insereMarcacoesPontoManual($dataset);

                    // insere marcação de saída
                    $dataset["dthrmarcacaoponto"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO_SAIDA"] . ":00";
                    $dataset["dthrajustada"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO_SAIDA"] . ":00";

                    $resultSai = $this->insereMarcacoesPontoManual($dataset);
                }
            }
        }

        if ($resultEnt && $resultSai)
            return true;
        else
            return false;

    }

    public function getHorarioBase($nrorg, $nrhordiam) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              TO_CHAR(TO_DATE(HRENTRADA, 'HH24MI') - 0.003472, 'HH24:MI') AS HRENTRADA_NOVA,
              SUBSTR(SUBSTR(HRENTRADA, 1, LENGTH(TRIM(HRENTRADA)) -2) || ':' || SUBSTR(HRENTRADA, -2), 1, 10) AS HRENTRADA,
              (SELECT SUBSTR(SUBSTR(HRSAIDA, 1, LENGTH(TRIM(HRSAIDA)) -2) || ':' || SUBSTR(HRSAIDA, -2), 1, 10) AS HRSAIDA
                 FROM GPE_INTERVALOSHORARIO IH
                WHERE IH.NRORG     = :P_NRORG
                  AND IH.NRHORDIAM = :P_NRHORDIAM
                  AND IH.NRSEQINTERVHORARIO = (SELECT MAX(NRSEQINTERVHORARIO)
                                                 FROM GPE_INTERVALOSHORARIO
                                                WHERE NRORG     = IH.NRORG
                                                  AND NRHORDIAM = IH.NRHORDIAM)) HRSAIDA
              FROM GPE_INTERVALOSHORARIO IH
             WHERE IH.NRORG     = :P_NRORG
               AND IH.NRHORDIAM = :P_NRHORDIAM
               AND IH.NRSEQINTERVHORARIO = (SELECT MIN(NRSEQINTERVHORARIO)
                                              FROM GPE_INTERVALOSHORARIO
                                             WHERE NRORG     = IH.NRORG
                                               AND NRHORDIAM = IH.NRHORDIAM)
        ");

        $query->execute(
            array(
                "P_NRORG"     => $nrorg,
                "P_NRHORDIAM" => $nrhordiam
            )
        );

        $horariobase = $query->fetchAll();

        if (count($horariobase) > 0) {
            return $horariobase[0];
        } else {
            return null;
        }
    }

    public function updateMarcacaoEntradaSaida($marcacoes) {
        $parameters = array();
        $parameters["P_DTAPURACAOPONTO"] = $marcacoes[0]["P_DTAPURACAOPONTO"];
        $parameters["P_NRMARCACAOPONTO"] = $marcacoes[0]["P_NRMARCACAOPONTO"];
        $parameters["P_IDFIXAENTSAID"]   = $marcacoes[0]["P_IDFIXAENTSAID"];
        $this->setIdfixaentsaidMarcacaoPonto($parameters);
    }

    public function getMarcacoesPontoDia($parameters) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS NRMARCACAOPONTO_LABEL,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO
             FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
               ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND NRVINCULOM = :P_NRVINCULOM
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA IN ('DA', 'DC', 'DS'))
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                   OR (IDDIAOCORRENCIA = 'FA' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA = 'FS' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY NRMARCACAOPONTO_LABEL, NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $parameters["P_NRORG"],
                "P_NRVINCULOM"      => $parameters["P_NRVINCULOM"],
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );

        $marcacoesPonto = $query->fetchAll();

        if (isset($marcacoesPonto[0])) {
            return $marcacoesPonto;
        } else {
            return null;
        }

    }

    public function reverterEntradaSaida($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
        ");

        $return = $sql->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );

        if ($return)
            return array('UPDATE' => true);
        else
            return array('UPDATE' => false);
    }
    
    public function getMarcacoesVinculoMesCompetencia($nrOrg, $dtIniStr, $dtFimStr){
        $query = $this->entityManager->getConnection()->prepare(
                 "SELECT NRVINCULOM, 
                         TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY HH24:MI') DTHRMARCACAOPONTO, 
                         IDNSRREP
                    FROM GPE_MARCACAOPONTO
                   WHERE NRORG = :P_NRORG
                     AND DTHRMARCACAOPONTO BETWEEN TO_DATE(:P_INI, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:P_FIM, 'DD/MM/YYYY HH24:MI:SS')"
        );
        
        $query->execute(
            array(
                "P_NRORG" => $nrOrg,
                "P_INI"   => $dtIniStr,
                "P_FIM"   => $dtFimStr
            )
        );
        
        $retorno = $query->fetchAll();
        return $retorno;
        
    } 
    
    public function importaArquivoParaMarcacao($row){
        set_time_limit(0);
        $nrorg = $row['NRORG'];
        $nrorg_padrao = $row['NRORG_PADRAO'];
        $mesCompetencia = $row['MESCOMPETENCIA'];
        $periodo = $row['NRPERIODOAPURACAO'];
        $arquivo = $row['DSPATH'][0];
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        if (!$file->isFile()) {
            throw new \Exception("Falha ao gravar o arquivo.");
        }
        
        $result =  $this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoIntervalo($nrorg, $periodo)[0];
        if (!$result) {
            throw new \Exception("Periodo não encontrado.");
        }
        
        if($result){
            $dtIniStr = $result['DTINICIALAPURACAO'].' 00:00:00';   // Adiquiri as datas de escopo e ajusta a hora limite do dia
            $dtFimStr = $result['DTFINALAPURACAO'].' 23:59:59';
            $dtIniObj = DateUtil::getDataDeString($dtIniStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);  // Converte as datas de escopo para objeto
            $dtFimObj = DateUtil::getDataDeString($dtFimStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);  // Converte as datas de escopo para objeto
            
            $tempoDescanco = $result['HRTEMPODESCANSODIARIO'];
        }
        
        $arquivo = file_get_contents($file->getPathname());
        $arquivo = str_replace("\r", '', $arquivo);
        $arquivo = explode("\n", $arquivo);
        
        $registros = array();
        $pisVinculoNaoEcontrado = '';
        
        foreach ($arquivo as $linha) {
            if($linha){
                $tpReg = substr($linha, 9, 1); // Tipo de registro 1: Cabeçalho, Tipo de registro 2: Inclusão ou Alteração da empresa no REP, Tipo de registro 3: Marcação de ponto
                if($tpReg == '3') {  // Somente são feitas buscas e adições de registros do tipo 3
                    $nsr = substr($linha, 0, 9);  // Número Sequencial de Registro
                    $data = substr($linha, 10,  8);
                    $hora = substr($linha, 18,  4);
                    $pis = substr($linha, 22, 12);
                    $dtMarcacaoStr = substr($data, 0, 2) . '/' . substr($data, 2, 2) . '/' . substr($data, 4, 4)  //Monta a data de marcação completa (em string). Parte da data (string)
                        . ' ' . substr($hora, 0, 2) . ':' . substr($hora, 2, 2) . ':00';                        //Parte da hora (string)
                    $dtMarcacaoObj = DateUtil::getDataDeString($dtMarcacaoStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    $vinculo = $this->gpeVinculomEntity->retornaVinculosAtivosMesCompetencia($nrorg, $mesCompetencia, null, null, null, false, null, true, false, null, null, null,  null, intval($pis));
                    if(count($vinculo) >= 1){
                        $vinculo = $vinculo[0];

                        if(DateUtil::getIntervaloEmSegundos($dtIniObj, $dtMarcacaoObj) >= 0 && DateUtil::getIntervaloEmSegundos($dtMarcacaoObj, $dtFimObj) > 0) {
                            $registros[$vinculo->getNrvinculom()][] = array(
                                'mar' => $dtMarcacaoObj,
                                'nsr' => $nsr);
                        }
                    } else{
                        $pisVinculoNaoEcontrado .= ''.$pis.',';
                    }
                }
            }
        }
        
        if($pisVinculoNaoEcontrado){
            $mensagem['erro'] = 'Vínculo não encontrado para o(s) pis: '.$pisVinculoNaoEcontrado.'.'; 
        }
        
        $marcacoes = $this->getMarcacoesVinculoMesCompetencia($nrorg, $dtIniStr, $dtFimStr);
        $listaMarcacoes = array();
        foreach($marcacoes as $marcacao){
            $listaMarcacoes[$marcacao['IDNSRREP']] = $marcacao;
        }
        
        $contVinc = 0;
        $contMarc = 0;
        foreach($registros as $vinculo => $registro){
            $marcAnterior  = null;
            $diaAnterior   = null;
            $idDia         = 'FC';
            $idDiaAnterior = null;
            
            foreach($registro as $valor){
                if(isset($listaMarcacoes[$valor['nsr']])){        
                    if($listaMarcacoes[$valor['nsr']]['DTHRMARCACAOPONTO'] == $valor['mar']->format('d/m/Y H:i')) {
                        continue;
                    }
                }
                
                $diaAtual = $valor['mar']->format('d/m/Y');

                if(isset($marcAnterior)){
                    // Condição para tornar a primeira marcação do dia como 'FA' caso a diferença dessa pra anterior seja inferior ao tempo de descanso (param. org.) e não seja no mesmo dia da anterior
                    $dif = $valor['mar']->diff($marcAnterior);
                    $dif = ($dif->s / 3600) + ($dif->i / 60) + $dif->h + ($dif->d * 24) + ($dif->m * 720) + ($dif->y * 8760);
                    if($dif > $tempoDescanco) {
                        $idDia = 'FC';
                    } else {
                        if($diaAtual == $diaAnterior) {
                            $idDia = $idDiaAnterior;
                        } else {
                            $idDia = 'FA';
                        }
                    }
                }
                
                // Busca horario do dia da marcação do vínculo. Se esse horário tiver virada, considerar as marcações do dia seguinte até 07 como FA
                // Isso é necessário pois existem horários que começam nesse periodo
                $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrorg, $vinculo, $diaAtual, $this->hcmParameters->getDtCompetencia());
                if (!empty($dadosHorarioVinculo)) {
                    $dadosHorarioIncluiDiaAnterior = $this->horarioRepository->horarioIncluiDiaAnterior($nrorg, $dadosHorarioVinculo['NRHORDIAM']);
                    $horarioIncluiDiaAnterior = !empty($dadosHorarioIncluiDiaAnterior) && $dadosHorarioIncluiDiaAnterior['response'];
                    
                    // Condição para que em escalas que 'viram' o dia, a primeira marcação seja considerada como 'FC' e as demais como 'FA'
                    // As marcações entre 00:00 e 07:00 são interpretadas como DA nesse cenário
                    // Essa condição sobrescreve o iddiaocorrencia setado anteriormente.
                    $dataInicioValidacaoDA  = $diaAtual . ' ' . self::HR_INICIO_VALIDACAO_MARC_DA;
                    $dataFimValidacaoDA     = $diaAtual . ' ' . self::HR_FIM_VALIDACAO_MARC_DA;
                    $marcInicioFA = DateUtil::getDataDeString($dataInicioValidacaoDA,   DateUtil::FORMATO_DATAHORA);
                    $marcLimiteFA = DateUtil::getDataDeString($dataFimValidacaoDA,      DateUtil::FORMATO_DATAHORA);
                    if ($horarioIncluiDiaAnterior && $valor['mar'] <= $marcLimiteFA && $valor['mar'] >= $marcInicioFA) {
                        $idDia = 'FA';
                    }
                }
                
                $marcAnterior  = $valor['mar'];
                $diaAnterior   = $marcAnterior->format('d/m/Y');
                $idDiaAnterior = $idDia;
                $nrMarcacaoPonto = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                $dtAtual = DateUtil::getDataAtual();
                
                $obj = new $this->gpeMarcacaoponto;  
                $obj->setNrvinculom($vinculo);                     
                $obj->setNrmarcacaoponto($nrMarcacaoPonto);    
                $obj->setNrorg($nrorg);
                $obj->setDthrmarcacaoponto($valor['mar']);
                $obj->setDthrajustada($valor['mar']);
                $obj->setIdnsrrep($valor['nsr']);
                $obj->setNrcoletor(24);
                $obj->setIddiaocorrencia($idDia);
                $obj->setIdenquadrahorario('N');
                $obj->setIdmarcdesprezada('N');
                $obj->setIdativo('S');
                $obj->setDtinclusao($dtAtual);
                $this->entityManager->persist($obj);
                $this->entityManager->flush($obj);
                $horarioMarcacoes[] = [$valor['mar'], $nrMarcacaoPonto, $idDia];

                $contMarc++;                                    
            }

            if (isset($horarioMarcacoes)) {
                $dataAtual = $horarioMarcacoes[0][0]; $contadorMarcacao = 0; $marcacao[$contadorMarcacao] = array("intervalo" => 0, "codigo" => null); $i = 1;
                // confere as marcações em pares para conferir qual intervalo é maior para se definir o horário de almoço
                for($i = 1; $i <= (count($horarioMarcacoes) - 1); $i++){
                    
                    if ($dataAtual->format('d/m/Y') === $horarioMarcacoes[$i][0]->format('d/m/Y') || 
                          ((isset($horarioMarcacoes[$i+1]) && ($horarioMarcacoes[$i+1][2] === "FA")) && 
                                    DateUtil::adicionaIntervalo($dataAtual)->format('d/m/Y') === $horarioMarcacoes[$i+1][0]->format('d/m/Y'))) {
                                        
                              
                        $intervalo = isset($horarioMarcacoes[$i+1]) ? DateUtil::getIntervaloEmSegundos($horarioMarcacoes[$i][0], $horarioMarcacoes[$i+1][0]) : null;
                        
                        if (isset($marcacao[$contadorMarcacao])  && $intervalo > $marcacao[$contadorMarcacao]["intervalo"]) {
                            $marcacao[$contadorMarcacao] = array("intervalo" => $intervalo, "codigo" => $horarioMarcacoes[$i][1]);
                        }
                        
                    } else if(DateUtil::getQtdeDiasEntre($dataAtual, $horarioMarcacoes[$i][0]) >= 1 && $horarioMarcacoes[$i][2] == "FC") {
                        $dataAtual = $horarioMarcacoes[$i][0];
                        $contadorMarcacao++;
                    }
                    $i++;
                }
                
                $alteEscala = $this->alteEscalaRepository->retornaUltimaAlteEscalaPorDataInicio($this->hcmParameters->getNrorgTrabalho(), $vinculo, DateUtil::getPrimeiroDiaDoMes($valor['mar'])->format('d/m/Y'));
                if (empty($alteEscala)) {
                    continue;
                }
                $alteEscala = $alteEscala[0];
                
                $horaDia =  $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(array("nrorg" => $nrorg, "nrhordiam" => $alteEscala['NRHORDIAM']));
                $nrintervalo = null;
                foreach($horaDia as $horario) {
                    if ($horario->getHrprojetarintervalo() !== null) {
                        $nrintervalo = $horario->getNrintervalohorario();
                        break;
                    }
                }
                
                // array para definir o id do dia da semana para a tabela
                $diaSemana = array('1' => '0100000',
                                   '2' => '0010000',
                                   '3' => '0001000',
                                   '4' => '0000100',
                                   '5' => '0000010',
                                   '6' => '0000001',
                                   '7' => '1000000');
                                   
                // percorre todos os intervalos de almoço salvos e salva uma ocorrência para cada um deles
                foreach($marcacao as $marc) {
                    $diaMarcacao = DateUtil::getDiaSemana($valor['mar']->format('d/m/Y'));
                    foreach($diaSemana as $key => $dia) {
                        if($key == $diaMarcacao){
                            $diaMarcacao = $dia;
                        }
                    }
                    
                    $nrmarcacaoponto = $marc["codigo"] ?? null;
                    
                    $ocorrencia = new $this->ptoProgOcorrencia;
                    $ocorrencia->setNrorg($nrorg);
                    $ocorrencia->setIdprogocorconfirmada('S');
                    $ocorrencia->setNrvinculom($vinculo);
                    $ocorrencia->setDtinicioprogocor($valor['mar']);
                    $ocorrencia->setDtfimprogocor($valor['mar']);
                    $ocorrencia->setIddiassemana($diaMarcacao);
                    $ocorrencia->setNrhordiam($alteEscala['NRHORDIAM']);
                    $ocorrencia->setNrintervalohorario($nrintervalo);
                    $ocorrencia->setIdtipohorario('N');
                    $ocorrencia->setNrtipoprogocorrencia(20);
                    $ocorrencia->setNrprogocorrencia(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_PROGOCORRENCIA', 1, 12, true));
                    $ocorrencia->setNrmarcacaoponto($nrmarcacaoponto);
                    $ocorrencia->setDtinclusao(DateUtil::getDataAtual());
                    $ocorrencia->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $ocorrencia->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                    $ocorrencia->setDtultatu(DateUtil::getDataAtual());
                    $ocorrencia->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $ocorrencia->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($ocorrencia);
                    $this->entityManager->flush($ocorrencia);
                }
                unset($horarioMarcacoes);
            }
            
            $contVinc++;                                       
        }
        
        $mensagem['sucesso'] = 'Importação de '.$contMarc.' marcações concluída para '.$contVinc.' vínculos.';
        
        return $mensagem;
        //Exibição de mensagens
        // $aux = 'Importação concluída.<br>Vinculos processados: '.$contVinc.'.<br>Marcações importadas: '.$contMarc.'.';
        // if(empty($msg)){
        //     tk_message($aux);
        // } else {
        //     tk_warning($aux.'<br>Avisos:<br>'.$msg);
        // }
    }
    
    public function retornaIdDIaSemana() {
        
    }
    
    
}