<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;

use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\Messages;

class OcorrenciaFrequenciaService {
    
    // ID de status da Programação de ocorrência
    protected const IDPROGOCORCONFIRMADA_PENDENTE   = 'P';
    protected const IDPROGOCORCONFIRMADA_CONFIRMADA = 'S';
    protected const IDPROGOCORCONFIRMADA_REJEITADA  = 'N';
    
    // Padrões de nome de arquivo
    protected const OCORRENCIA_DOWNLOAD_FILE_NAME           = 'DOCUMENTO_OCORRENCIA';
    protected const OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR = '_';
    
    protected const AVAILABLE_IDPROGOCORCONFIRMADA  = array(self::IDPROGOCORCONFIRMADA_PENDENTE, self::IDPROGOCORCONFIRMADA_CONFIRMADA, self::IDPROGOCORCONFIRMADA_REJEITADA);

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager    = $entityManager;
        $this->hcmParameters    = $hcmParameters;
    }
    
    /**
     * Rotina responsável pela alteração de status de ocorrências de frequências (tabela PTO_PROGOCORRENCIA). Disponíveis: 'Pendente', 'Confirmada' e 'Rejeitada'.
     * 
     *  @param $nrprogocorrencia
     *  @param $idprogocorconfirmada
     *  @param $dsobservacaorecusa
     */
    public function alteraStatusOcorrencia($nrprogocorrencia, $idprogocorconfirmada, $dsobservacaorecusa = null) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        // Valida se o status existe
        $idProgOcorNaoExiste = !in_array($idprogocorconfirmada, self::AVAILABLE_IDPROGOCORCONFIRMADA);
        if ($idProgOcorNaoExiste) {
            return false;
        }
        
        // Busca a ocorrência
        $ptoProgOcorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrOrg, 'nrprogocorrencia' => $nrprogocorrencia));
        if (empty($ptoProgOcorrencia)) {
            return false;
        }
        
        // Altera o status
        $ptoProgOcorrencia->setIdprogocorconfirmada($idprogocorconfirmada);
        
        // Altera a descrção de recusa em caso de rejeição da ocorrência. Se não, apaga a descrição.
        if ($idprogocorconfirmada === self::IDPROGOCORCONFIRMADA_REJEITADA) {
            $ptoProgOcorrencia->setDsobservacaorecusa($dsobservacaorecusa);
        } else {
            $ptoProgOcorrencia->setDsobservacaorecusa(null);
        }
        
        $this->entityManager->flush();
        
        return true;
    }
    
    /**
     *  Rotina responsável pela donwload dos documentos associados à ocorrências de frequências (tabela PTO_PROGOCORRENCIA)
     * 
     *  @param $nrprogocorrencia
     * 
     *  @return $filePath
     */
    public function downloadArquivosOcorrencia($nrprogocorrencia) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        // Busca a ocorrência
        $ptoProgOcorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrOrg, 'nrprogocorrencia' => $nrprogocorrencia));
        if (empty($ptoProgOcorrencia)) {
            throw new \Exception(Messages::HCM07018_PTO_OCORR_NOT_FOUND);
        }
        
        $dsPathOcorrencia = $ptoProgOcorrencia->getDspath();
        
        if (empty($dsPathOcorrencia)) {
            throw new \Exception(Messages::HCM07018_PTO_OCORR_NO_FILE);
        }
        
        $outputFileName = $this->generateDownloadedOcorrenciaFileName($nrprogocorrencia);
        $filePath = $this->util->getNewFileServerFile($dsPathOcorrencia, $outputFileName);
        
        return $filePath;
    }
    
    /**
     *  Rotina responsável por gerar o nome do arquivo relacionado a ocorrência que será baixado a partir dos parâmetros OCORRENCIA_DOWNLOAD_FILE_NAME, OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR, $nrprogocorrencia e a data atual
     * 
     *  @param $nrprogocorrencia
     * 
     *  @return $fileName
     */
    public function generateDownloadedOcorrenciaFileName($nrprogocorrencia) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $fileDate = date("YmdHis");
        
        $fileName = self::OCORRENCIA_DOWNLOAD_FILE_NAME . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $nrOrg . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $nrprogocorrencia . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $fileDate;
        
        return $fileName;
    }
    
}