<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Folha\Entities\GpeReuniaoparticip;
use Zeedhi\Framework\DTO;
use HCM\Util\Messages;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ReuniaoController extends Crud {
    
    protected $reuniaoService;
    protected $dataSourceName = 'gpe_reuniao';
    protected $dataSourceName1 = 'gpe_reuniao_get';
    protected $dataSourceName2 = 'gpe_reuniaoparticip_get';
    protected $dataSourceName3 = 'gpe_reuniaoitem_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'IDPAUTADELIBERACAO');
    const CADASTRO_AUTOMATICO = 'cadastro_automatico';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\ReuniaoService $reuniaoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->reuniaoService = $reuniaoService;
        parent::__construct($dataSourceManager);
    }

    public function cadastroAutomaticoParticipanteReuniao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $action = $row['action'];
                $participantes_list = $row['NRPARCNEGOCIO_LIST'];
                $compareceu = $row['IDPARTICIPCOMPARECEU'];
                $nrreuniao = $row['NRREUNIAO'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
                switch($action) {
                    case self::CADASTRO_AUTOMATICO:
                        $this->reuniaoService->cadastroAutomatico($nrreuniao,$participantes_list,$compareceu,$nrorg,$nrorgpadrao,$dtmescompetenc);
                        break;
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getReuniao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setReuniao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRREUNIAO', $row)) {
                    if ($row['NRREUNIAO'] === '') {
                        $row['NRREUNIAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAO', 1, 12, true);
                    }
                } else {
                    $row['NRREUNIAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAO', 1, 12, true);
                }
                if(!array_key_exists('NRESTRUTURAM', $row)){
                    $row['NRESTRUTURAM'] = null;
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteReuniao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->reuniaoService->deleteReuniao($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function getParticipaReuniao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setParticipaReuniao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_reuniaoparticip";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if($row['__is_new'] === true){
                    $participantes = GpeReuniaoparticip::findOneByNrreuniaoAndNrorgAndNrparcnegocio($row['NRREUNIAO'], $row['NRORG'], $row['NRPARCNEGOCIO']);
                    if(isset($participantes)){
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM05000_SST_REUNIAO_EXISTS_PARTICIPANTESCADASTRADO_ERROR, DTO\Response\Message::TYPE_ERROR));
                    }
                }    
                if (array_key_exists('NRREUNIAOPARTICIP', $row)) {
                    if ($row['NRREUNIAOPARTICIP'] === '') {
                        $row['NRREUNIAOPARTICIP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAOPARTICIP', 1, 12, true);
                    }
                } else {
                    $row['NRREUNIAOPARTICIP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAOPARTICIP', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteParticipaReuniao(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "gpe_reuniaoparticip";
        parent::delete($request, $response);
    }
    
    public function getReuniaoItem(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setReuniaoItem(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_reuniaoitem";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRREUNIAOITEM', $row)) {
                    if ($row['NRREUNIAOITEM'] === '') {
                        $row['NRREUNIAOITEM'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAOITEM', 1, 12, true);
                    }
                } else {
                    $row['NRREUNIAOITEM'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_REUNIAOITEM', 1, 12, true);
                }
                if(!array_key_exists('NRACAO', $row)){
                    $row['NRACAO'] = null;
                }
                if(!array_key_exists('DTPREVISTADELIB', $row)){
                    $row['DTPREVISTADELIB'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteReuniaoItem(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "gpe_reuniaoitem";
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 