<?php
/**
 * FPARegistro.
 *
 * @package Logic
 * @author  alessandrobartels - 02/06/2011
 */
abstract class FPARegistro{

    const PREENCHE_DIREITA = STR_PAD_RIGHT;
    const PREENCHE_ESQUERDA = STR_PAD_LEFT;

    // Formato de quebra de linha para padrao Windows (CR -> retorno da carro | LF -> quebra de linha ).
    const QUEBRA_LINHA_CR_LF = "\r\n";

    const CARACTER_ACENTUADO =     'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ';
    const CARACTER_NAO_ACENTUADO = 'AAAAAAACEEEEIIIIDNOOOOOOUUUUYBSAAAAAAACEEEEIIIIDNOOOOOOUUUYYBY';
    const APENAS_CARACTERES_ALFANUMERICOS = '/[^a-zA-Z0-9\s]/';
    const CARACTER_ASTERISCO = '*';
    const CARACTER_PONTO = '.';
    const CARACTER_PARENTESE_ESQ = '(';
    const CARACTER_PARENTESE_DIR = ')';
    const CARACTER_VIRGULA = ',';
    const CARACTER_BRANCO = ' ';
    const CARACTER_VAZIO = '';
    const CARACTER_ZERO = '0';
    const CARACTER_UM = '1';
    const CARACTER_DOIS = '2';
    const CARACTER_NOVE = '9';
    const CARACTER_ONZE = '11';
    const CARACTER_DOZE = '12';
    const CARACTER_C = 'C';
    const CARACTER_UNDERLINE = '_';


    private $_linhaRegistro;


    /**
     * Contrato - Gera Registro.
     *
     * @param string $tpRegistro  Numero do Tipo de Registro.
     * @param string $tpInscricao Numero do Tipo de Inscricao.
     * @param string $nrInscricao Numero da Inscricao.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    protected abstract function geraRegistro($tpRegistro, $tpInscricao, $nrInscricao);


    /**
     * Acesso ao atributo linhaRegistro.
     *
     * @return string
     *
     * @author alessandrobartels - 02/06/2011
     */
    public function getLinhaRegistro(){
        return $this->_linhaRegistro;
    }


    /**
     * Seta valor ao atributo linhaRegistro.
     *
     * @param string $conteudo Conteudo da linha.
     *
     * @return void
     *
     * @author alessandrobartels - 13/06/2011
     */
    public function setLinhaRegistro($conteudo = null){
        $this->_linhaRegistro = $conteudo;
    }


    /**
     * Remove acentos e caracteres alfa especiais.
     *
     * @param string $string String a ser manipulada.
     *
     * @return string
     *
     * @author alessandrobartels - 02/06/2011
     */

    private static function _removeAcentos($string){
        //$string = utf8_decode($string);
        /* $string = strtr($string, utf8_decode(self::CARACTER_ACENTUADO), self::CARACTER_NAO_ACENTUADO); */
        $string = strtr($string, self::CARACTER_ACENTUADO, self::CARACTER_NAO_ACENTUADO);
        /* return utf8_encode($string); */
        return $string;

    }


    /**
     * Remove caracter especiais.
     *
     * @param string $string String a ser manipulada
     *
     * @return string
     *
     * @author alessandrobartels - 15/05/2012
     *
     */
    private static function _removeCaracteresEspeciais($string){
//        $string = str_replace(self::CARACTER_PONTO, self::CARACTER_VAZIO, $string);
//        $string = str_replace(self::CARACTER_VIRGULA, self::CARACTER_VAZIO, $string);
//        $string = str_replace(self::CARACTER_PARENTESE_ESQ, self::CARACTER_VAZIO, $string);
//        $string = str_replace(self::CARACTER_PARENTESE_DIR, self::CARACTER_VAZIO, $string);
        $string = preg_replace(self::APENAS_CARACTERES_ALFANUMERICOS, self::CARACTER_VAZIO, $string);
        $string = str_replace('  ', ' ',$string);

        return $string;
    }


    /**
     * Faz tratamento da string.
     *
     * @param string $dado String que sera tratada.
     *
     * @return string
     *
     * @author thalesvalias - 04/04/2012
     */
    public static function _trataDado($dado, $decimal = false) {
        if (empty($dado) === false) {
            $dado = Valor::atribuiCasaDecimal($dado);
            if ($decimal !== true) {$dado = self::_removeCaracteresEspeciais($dado);}
            $dado = self::_removeAcentos($dado);
            $dado = strtoupper($dado);
        }

        return $dado;
    }


    private function _truncaDado($dado, $tamanhoCampo, $start){
        return substr($dado, $start, $tamanhoCampo);
    }


    /**
     * Concatena dados no atributo linhaRegistro.
     *
     * @param string  $dado         String que sera armazenada.
     * @param integer $tamanhoCampo Tamanho do campo.
     * @param string  $caracter     Caracter para preenchimento do campo caso a string seje menor.
     * @param string  $orientacao   Orientacao do preenchimento (esquerda ou direita).
     * @param integer $start        Posicao inicial.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     *         thalesvalias - 04/04/2012 - Retirada o tratamento de string e criado metodo trataDado().
     */
    protected function atribuiDado($dado, $tamanhoCampo, $caracter = null, $orientacao = null, $start = 0){
        $dado = $this->_trataDado($dado);
        $dado = $this->_truncaDado($dado, $tamanhoCampo, $start);
        $this->_linhaRegistro .= str_pad($dado, $tamanhoCampo, $caracter, $orientacao);
    }

    /* Concatena dados no atributo linhaRegistro.
     * @param string  $dado         String que sera armazenada.
     * @param integer $tamanhoCampo Tamanho do campo.
     * @param string  $caracter     Caracter para preenchimento do campo caso a string seje menor.
     * @param string  $orientacao   Orientacao do preenchimento (esquerda ou direita).
     * @param integer $start        Posicao inicial.
     * @return void
     * @author alessandrobartels - 02/06/2011
     *         thalesvalias - 04/04/2012 - Retirada o tratamento de string e criado metodo trataDado().
     */

    protected function atribuiDadoEspecial($dado, $tamanhoCampo, $caracter = null, $orientacao = null, $start = 0){

        //$dado = $this->_trataDado($dado);
        $dado = $this->_truncaDado($dado, $tamanhoCampo, $start);
        $this->_linhaRegistro .= str_pad($dado, $tamanhoCampo, $caracter, $orientacao);

    }


    protected function atribuiDadoXML($tag, $separador, $dado, $tamanhoCampo, $decimal = false, $start = 0, $manipula = true){

        if ((empty($dado) === false) && ($manipula === true)){
            $dado = $this->_trataDado($dado, $decimal);
            $dado = $this->_truncaDado($dado, $tamanhoCampo, $start);
         //   $dado = str_pad($dado, $tamanhoCampo, $caracter, $orientacao);
        }
        $elemento = $tag.$dado.$separador;
        $this->_linhaRegistro .= $elemento;
    }


    /**
     * Substitui um dado no registro em uma posicao especifica.
     *
     * @param string  $registro       String fonte.
     * @param string  $dado           String que a ser substituida.
     * @param integer $posicaoInicial String que sera armazenada.
     * @param integer $tamanhoCampo   Tamanho do campo.
     * @param string  $caracter       Caracter para preenchimento do campo caso a string seje menor.
     * @param string  $orientacao     Orientacao do preenchimento (esquerda ou direita).
     *
     * @return string
     *
     * @author thalesvalias - 04/04/2012
     */
    public function alteraDado($registro, $dado, $posicaoInicial, $tamanhoCampo, $caracter, $orientacao){

        $dado = $this->_trataDado($dado);
        $dado = str_pad($dado, $tamanhoCampo, $caracter, $orientacao);
        $linha = substr_replace($registro, $dado, $posicaoInicial, $tamanhoCampo);

        return $linha;
    }

    /**
     * Faz tratamento da string.
     * @param string $dado String que sera tratada.
     *
     * @return string
     *
     * @author: Matheus Silva
     */
    public static function _trataDadoRais($dado) {
        if (empty($dado) === false) {
            $dado = self::_removeCaracteresEspeciais($dado);
            $dado = self::_removeAcentos($dado);
            $dado = strtoupper($dado);
        }
        return $dado;
    }


}